#include <dlfcn.h>
#include <graph/server/plugin.h>
#include <stdexcept>
#include <string>

namespace Graph {
Plugin::Plugin(const std::string& path) : path(path) {
  this->handle = dlopen(path.c_str(), RTLD_LAZY);
  if (!this->handle)
    throw std::runtime_error(dlerror());

  this->plugin = (plugin_t*)dlsym((void*)this->handle, "plugin");
  if (!this->plugin)
    throw std::runtime_error(dlerror());

  this->plugin->update_callback = nullptr;
}

Plugin::~Plugin() {
  if (this->plugin->update_callback)
    this->plugin->destroy_method();

  if (this->handle)
    dlclose((void*)this->handle);
}

bool Plugin::init(update_callback_t callback) {
  this->plugin->update_callback = callback;
  return this->plugin->init_method();
}

PluginModel::~PluginModel() {
  for (Plugin* plugin : this->plugins) {
    delete plugin;
  }
}

int PluginModel::rowCount(const QModelIndex& parent) const {
  return this->plugins.size();
}

QVariant PluginModel::data(const QModelIndex& index, int role) const {
  if (!index.isValid())
    return QVariant();

  if (role == Qt::DisplayRole) {
    return QString(plugins[index.row()]->get_path().c_str());
  }

  return QVariant();
}

void PluginModel::add(Plugin* plugin) {
  plugins << plugin;
}

void PluginModel::remove(int index) {
  delete plugins[index];
  plugins.removeAt(index);
}

} // namespace Graph